use chromium::rocket::Chromium;
use nanobyte_opentelemetry::rocket::TracingFairing;
use rocket::{Rocket, Build, routes, fairing::AdHoc};
use rocket_prometheus::PrometheusMetrics;
use serde::Deserialize;
use tools::cache::{build_cache, cache_prometheus_metrics};
pub mod routes;
mod chromium;
mod tools;
mod services;



#[derive(Deserialize)]
#[serde(crate = "rocket::serde")]
pub struct CVBackendConfig {
    cv_backend_path: String,
}


#[derive(Deserialize)]
#[serde(crate = "rocket::serde")]
#[allow(dead_code)]
pub struct DefaultPerson {
    pub(crate) default_person_name: String,
}

pub fn rocket_builder() -> Rocket<Build> {
    let prometheus = PrometheusMetrics::new();
    cache_prometheus_metrics(&prometheus).expect("Failed to register prometheus cache metrics");

    rocket::build()
        .attach(TracingFairing::ignite())
        .attach(Chromium::ignite())
        .attach(
            AdHoc::config::<CVBackendConfig>()
        )
        .attach(
            AdHoc::config::<DefaultPerson>()
        )
        .attach(prometheus.clone())
        .manage( build_cache())
        .mount("/", routes![
            routes::pdf::render_pdf_cv,
            routes::pdf::render_html_cv
        ])
        .mount("/metrics", prometheus)
}
